module Soundyoink
  # Manages a queue of Audio instances to be downloaded, and knows how to
  # create many Audio instances from a user profile link.
  class Downloader
    # @param workers [Integer] Number of download threads to run in parallel.
    def initialize(workers: 1)
      @audios = Queue.new
      @workers = workers
    end

    # If given a link to an audio page, adds it to the queue.
    # If given a user profile link, adds each individual audio to the queue.
    # @param link [String] Link to either an individual audio or a user profile.
    def add(link)
      if link =~ %r{^https://.+[.net]/u/[^/]+$}
        add_profile(link)
      else
        @audios << Audio.new(link)
      end
    end

    # Find every audio link on a user's profile and add them to the queue.
    # @param profile [String] Link to a user's profile page.
    def add_profile(profile)
      open(profile).read
                   .scan(%r{(https://[^.]+[.]net/u/[^"]+)})
                   .flatten
                   .map { |a| @audios << Audio.new(a) }
    end

    # +@workers+ number of threads work on the +@audios+ queue in parallel.
    # @see Soundyoink::Audio#download
    def download
      worker_threads = []
      @workers.times do
        worker_threads << Thread.new do
          @audios.pop.download until @audios.empty?
        end
      end
      worker_threads.map(&:join)
    end
  end
end
